unit Delta.FMX.Controls;

interface

uses System.Classes, System.SysUtils, System.Types, System.UITypes,
  System.TypInfo, System.rtti, FMX.Graphics, FMX.Controls;

type
  TOnPaintEventCallBack = procedure(SenderRef, CanvasRef, ARectRef: Integer;
    DReference, DFuncPointer: Integer); stdcall;

procedure registerOnPaintEvent(Reference: Integer; AName: PAnsiChar;
  DReference, DFuncPointer: Integer; CallBack: TOnPaintEventCallBack); stdcall;

type
  TOnPaintEventWrapper = class(TComponent)
  private
    FCallBack: TOnPaintEventCallBack;
    FDReference: Integer;
    FDFuncPointer: Integer;
  public
    constructor Create(Owner: TComponent; DReference, DFuncPointer: Integer;
      CallBack: TOnPaintEventCallBack); reintroduce;
  published
    procedure Event(Sender: TObject; Canvas: TCanvas; const ARect: TRectF);
  end;

exports registerOnPaintEvent;

implementation

uses Delta.Properties;

procedure registerOnPaintEvent(Reference: Integer; AName: PAnsiChar;
  DReference, DFuncPointer: Integer; CallBack: TOnPaintEventCallBack); stdcall;
var
  nEvent: TOnPaintEvent;
  component: TComponent;
begin
  if Assigned(CallBack) then
  begin
    try
      component := TComponent(Reference);
      nEvent := TOnPaintEventWrapper.Create(component, DReference, DFuncPointer,
        CallBack).Event;
      setPropertyValue(Reference, string(AName),
        TValue.From<TOnPaintEvent>(nEvent));
    except
    end;
  end
  else
  begin
    setPropertyValue(Reference, string(AName), nil);
  end;
end;

{ TOnPaintEventWrapper }

constructor TOnPaintEventWrapper.Create(Owner: TComponent;
  DReference, DFuncPointer: Integer; CallBack: TOnPaintEventCallBack);
begin
  inherited Create(Owner);
  FCallBack := CallBack;
  FDReference := DReference;
  FDFuncPointer := DFuncPointer;
end;

procedure TOnPaintEventWrapper.Event(Sender: TObject; Canvas: TCanvas;
  const ARect: TRectF);
begin
  FCallBack(Integer(Sender), Integer(Canvas), Integer(@ARect), FDReference,
    FDFuncPointer);
end;

end.
