module FMX.Forms;

import delta.core;
import delta.core.events;
import System.Classes;
import FMX.Types: TFmxObject;
import FMX.Graphics: TBrushKind, TBrush;
import System.UITypes: TCloseAction, TModalResult;

alias TIdleEvent = void delegate(TObject sender, ref bool done);

private TApplication _application;

@property TApplication Application()
{
	if (_application is null)
	{
		int r = executeClassMethodReturnReferenceArgsNone("Delta.FMX.Forms.TForms", "GetApplication");
		_application = new TApplication(r);
	}
	return _application;
}

class TApplication: TComponent
{	
	alias reference this;
	
	mixin(GenObjectProperty!("MainForm", TCommonCustomForm));
	mixin(GenIdleEvent!"OnIdle");
	
	
	this(int reference)
	{
		super(reference);
	}
	
	void Initialize()
	{
		executeInstanceMethodReturnNoneArgsNone(this, "Initialize");
	}
	
	void Run()
	{
		executeInstanceMethodReturnNoneArgsNone(this, "Run");
	}
	
	void Terminate()
	{
		executeInstanceMethodReturnNoneArgsNone(this, "Terminate");
	}
}


alias TCloseEvent = void delegate(TObject sender, ref TCloseAction action);
alias TCloseQueryEvent = void delegate (TObject sender, ref bool canClose);

enum TDeviceKind {
	Desktop, 
	iPhone, 
	iPad
};

enum TFormPosition {Designed, Default, DefaultPosOnly, DefaultSizeOnly, ScreenCenter, DesktopCenter, MainFormCenter, OwnerFormCenter};

alias TDeviceKinds = TDeviceKind[]; // TODO Set of TDeviceKinds

class TFormFactor: TPersistent
{
	mixin(GenSetProperty!("Devices", TDeviceKind));
	mixin(GenIntProperty!("Height"));
	mixin(GenIntProperty!("Width"));
	
	this(int reference)
	{
		super(reference);
	}
}

class TCommonCustomForm: TFmxObject
{	
	mixin(GenStringProperty!("Caption"));
	mixin(GenIntProperty!("ClientHeight"));
	mixin(GenIntProperty!("ClientWidth"));
	mixin(GenObjectProperty!("FormFactor", TFormFactor));
	mixin(GenIntProperty!("Left"));
	mixin(GenIntProperty!("Top"));
	mixin(GenIntProperty!("Width"));
	mixin(GenIntProperty!("Height"));
	mixin(GenBoolProperty!("Visible"));
	mixin(GenEnumProperty!("Position", TFormPosition));
	
	mixin(GenNotifyEvent!"OnCreate");
	mixin(GenNotifyEvent!"OnActivate");
	mixin(GenNotifyEvent!"OnShow");
	//mixin(GenEvent!("OnClose", TCloseEvent));
	
	void Close()
	{
		executeInstanceMethodReturnNoneArgsNone(this.reference, "Close");
	}
	
	void Show()
	{
		executeInstanceMethodReturnNoneArgsNone(this.reference, "Show");
	}

	TModalResult ShowModal()
	{
		return executeInstanceMethodReturnIntArgsNone(this.reference, "ShowModal");
	}
	
	this(int reference)
	{
		super(reference);
	}
	
	static TCommonCustomForm Create(TComponent owner)
	{
		int ownerReference = owner is null ? nil : owner.reference;
		int reference = executeClassMethodReturnReferenceArgsReference("FMX.Forms.TCommonCustomForm", "Create", ownerReference);
		return new TCommonCustomForm(reference);
	}

	static TCommonCustomForm CreateNew(TComponent owner, ptrdiff_t Dummy = 0)
	{
		int ownerReference = owner is null ? nil : owner.reference;
		int reference = executeClassMethodReturnReferenceArgsReferenceInt("FMX.Forms.TCustomForm", "CreateNew", ownerReference, Dummy);
		return new TCommonCustomForm(reference);
	}
}

class TCustomForm: TCommonCustomForm
{	
	this(TComponent owner)
	{
		int ownerReference = owner is null ? nil : owner.reference;
		int dummy;
		int reference = executeClassMethodReturnReferenceArgsReferenceInt("FMX.Forms.TCustomForm", "CreateNew", ownerReference, dummy);
		super(reference);
	}
	
	this(int reference)
	{
		super(reference);
	}
	
	static TCustomForm Create(TComponent owner)
	{
		int ownerReference = owner is null ? nil : owner.reference;
		int reference = executeClassMethodReturnReferenceArgsReference("FMX.Forms.TCustomForm", "Create", ownerReference);
		return new TCustomForm(reference);
	}
}

class TForm: TCustomForm
{	
	this(int reference)
	{
		super(reference);
	}
	
	static TForm Create(TComponent owner)
	{
		int ownerReference = owner is null ? nil : owner.reference;
		int reference = executeClassMethodReturnReferenceArgsReference("FMX.Forms.TForm", "Create", ownerReference);
		return new TForm(reference);
	}
}