module FMX.Graphics;

import delta.core;

import System.Classes;
import System.UITypes;
import System.Types;
import FMX.Types;

import FMX.Types: IFreeNotification;

enum TBrushKind { None, Solid, Gradient, Bitmap, Resource }

class TBitmap : TInterfacedPersistent, IStreamPersist
{
	mixin DelphiClass!("FMX.Types.TTimer");
	mixin(GenStructProperty!("BoundsF", TRectF));

	static TBitmap Create();

	static TBitmap CreateFromFile(string AFileName);

	bool IsEmpty();

	void LoadFromStream(TStream Stream);

	void SaveToStream(TStream Stream);

	void LoadFromFile(string AFileName);
}

class TCanvas : TInterfacedPersistent
{
	mixin(GenObjectProperty!("Stroke", TStrokeBrush));
	mixin(GenObjectProperty!("Fill", TBrush));

	this(int reference)
	{
		super(reference);
	}

	void DrawLine(TPointF APt1, TPointF APt2, float AOpacity)
	{
		executeInstanceMethodReturnNoneArgsStructStructFloat(this.reference, "DrawLine", cast(int) &APt1, cast(int) &APt2, AOpacity);
	}

	void DrawLine(TPointF APt1, TPointF APt2, float AOpacity, TStrokeBrush ABrush)
	{
		executeInstanceMethodReturnNoneArgsStructStructFloatReference(this.reference, "DrawLine", cast(int) &APt1, cast(int) &APt2, AOpacity, ABrush.reference);
	}

	void DrawEllipse(TRectF ARect, float AOpacity)
	{
		executeInstanceMethodReturnNoneArgsStructFloat(this.reference, "DrawEllipse", cast(int) &ARect, AOpacity);
	}

	void FillEllipse(TRectF ARect, float AOpacity)
	{
		executeInstanceMethodReturnNoneArgsStructFloat(this.reference, "FillEllipse", cast(int) &ARect, AOpacity);
	}

	void DrawBitmap(TBitmap ABitmap, TRectF SrcRect, TRectF DstRect, float AOpacity, bool HighSpeed)
	{
		executeInstanceMethodReturnNoneArgsReferenceStructStructFloatBoolean(this.reference, "DrawBitmap", ABitmap.reference,  cast(int) &SrcRect,  cast(int) &DstRect, AOpacity, HighSpeed);
	}

	void DrawPath(TPathData APath, float AOpacity)
	{
		executeInstanceMethodReturnNoneArgsReferenceFloat(this.reference, "DrawPath", APath.reference, AOpacity);
	}
}

class TBrush: TPersistent
{
	mixin(GenUIntProperty!("Color"));
	mixin(GenEnumProperty!("Kind", TBrushKind));
	mixin(GenObjectProperty!("Gradient", TGradient));
	
	this(int reference)
	{
		super(reference);
	}
}

class TStrokeBrush: TBrush
{
	this(int reference)
	{
		super(reference);
	}

	static TStrokeBrush Create(TBrushKind ADefaultKind, TAlphaColor ADefaultColor)
	{
		int reference = executeClassMethodReturnReferenceArgsIntUInt("FMX.Graphics.TStrokeBrush", "Create", ADefaultKind, ADefaultColor);
		return new TStrokeBrush(reference);
	}
}

class TGradientPoint: TCollectionItem
{
	mixin(GenUIntProperty!("Color"));
	mixin(GenFloatProperty!("Offset"));
	
	this(int reference)
	{
		super(reference);
	}
}

class TTextSettings: TPersistent
{
	mixin(GenUIntProperty!("FontColor"));
	mixin(GenObjectProperty!("Font", TFont));
	
	package this(int reference)
	{
		super(reference);
	}
}

class TFont: TPersistent
{
	mixin(GenFloatProperty!("Size"));
	mixin(GenSetProperty!("Style", TFontStyle));
	
	package this(int reference)
	{
		super(reference);
	}
}

class TGradientPoints: TCollection
{
	this(int reference)
	{
		super(reference);
	}
	
	TGradientPoint opIndex(size_t index)
	{
		// TODO
		return null;
	}
}

class TGradient: TPersistent
{
	mixin(GenObjectProperty!("Points", TGradientPoints));

	this(int reference)
	{
		super(reference);
	}
}

class TPathData : TInterfacedPersistent, IFreeNotification
{
	mixin(GenStringProperty!("Data"));
	
	this(int reference)
	{
		super(reference);
	}

	static TPathData Create()
	{
		int reference = executeClassMethodReturnReferenceArgsNone("FMX.Graphics.TPathData", "Create");
		return new TPathData(reference);
	}

	void FreeNotification(TObject AObject)
	{
		executeInstanceMethodReturnNoneArgsReference(this.reference, "FreeNotification", AObject.reference);
	}
}

interface IBitmapObject : IFreeNotificationBehavior
{
    //TBitmap GetBitmap(); Not a public function
    TBitmap Bitmap();
}
