module FMX.Types;

// Word => ushort

import delta.core;
import System.Classes, System.Generics.Collections;

alias TKeyEvent = void delegate(TObject sender, out ushort key, out wchar keyChar, TShiftState shift);

enum TStyledSetting  {Family, Size, Style, FontColor, Other};

enum TAlignLayout {None, Top, Left, Right, Bottom, MostTop, MostBottom, MostLeft, MostRight, Client, Contents, Center, VertCenter, HorzCenter, Horizontal, Vertical, Scale, Fit, FitLeft, FitRight};

class TFmxChildrenList: TEnumerable!TFmxObject
{
	this(int reference)
	{
		super(reference);
	}
}

class TFmxObject: TComponent
{
	mixin(GenObjectProperty!("Parent", TFmxObject));
	mixin(GenObjectProperty!("tagObject", TObject));
	mixin(GenStringProperty!("TagString"));
	mixin(GenFloatProperty!("TagFloat"));
	mixin(GenObjectProperty!("Children", TFmxChildrenList));

	this(int reference)
	{
		super(reference);
	}
	
	void AddObject(TFmxObject AObject)
	{
		executeInstanceMethodReturnNoneArgsReference(this.reference, "AddObject", AObject.reference);
	}
	
	void DeleteChildren()
	{
		executeInstanceMethodReturnNoneArgsNone(this.reference, "DeleteChildren");
	}
	
	void AddFreeNotify(const IFreeNotification AObject)
	{
		// to be implemented
	}

	void RemoveFreeNotify(const IFreeNotification AObject)
	{
		// to be implemented
	}

	
}

class TPosition: TPersistent
{
	mixin(GenBoolProperty!("StoreAsInt"));
	mixin(GenFloatProperty!("X"));
	mixin(GenFloatProperty!("Y"));
	
	this(int reference)
	{
		super(reference);
	}
}

class TBounds: TPersistent
{
	mixin(GenFloatProperty!("Bottom"));
	mixin(GenFloatProperty!("Left"));
	mixin(GenFloatProperty!("Right"));
	mixin(GenFloatProperty!("Top"));
	
	this(int reference)
	{
		super(reference);
	}
}

interface IFreeNotification 
{
	void FreeNotification(TObject AObject);
}

interface IFreeNotificationBehavior
{
    void AddFreeNotify(const IFreeNotification AObject);
    void RemoveFreeNotify(const IFreeNotification AObject);
}